/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Webserver uvm.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include <avr/io.h>
#include "config.h"
#include "usart.h"
#include "enc28j60.h"
#include "stack.h"
#include "timer.h"
#include "httpd.h"
#include "telnetd.h"
#include "cmd.h"
#include "ntp.h"
#include "base64.h"
#include "analog.h"
#include "zabex.h"
#include "memcheck.h"
#include <stdio.h>
//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void){  
  int i;
  asm("WDR"); //Reset Watchdog
  #if defined (__AVR_ATmega644__) || defined (__AVR_ATmega644P__)
  WDTCSR = 7;
  #else  
  WDTCR = 7; //2s Timeout
  #endif 

	//Konfiguration der Ausgnge bzw. Eingnge
	//definition erfolgt in der config.h
	DDRA = OUTA;
	DDRC = OUTC;
	DDRD = OUTD;
	
  unsigned long a;
  usart_init(BAUDRATE); // setup the UART
	
  ADC_Init();
	
    usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
    usart_write("Compiliert mit GCC Version "__VERSION__"\r\n");
	for(a=0;a<1000000;a++){asm("nop");};

	//Applikationen starten
	stack_init();

	httpd_init();
	initHistorie();
  
	//Ethernetcard Interrupt enable
	ETH_INT_ENABLE;
	
	//Globale Interrupts einschalten
	sei(); 

  asm("WDR"); //Reset Watchdog
  ntp_init();
  ntp_request();
	
	gas_l = 90;	
	gas_h = 120;	
	wasser_l = 180;	
	wasser_h = 195;	
	strom_l = 20;	
	strom_h = 70;	


  ANALOG_ON;
  
  for (i=0; i<10; GETwerte[i++]=0)
  ;
  jetzt.Zaehlerstand_Gas=0;    //TODO: Auslesen aus EEPROM
  jetzt.Zaehlerstand_Wasser=0;
  jetzt.Zaehlerstand_Strom=0;
  Basis.Zaehlerstand_Gas=0;
  Basis.Zaehlerstand_Wasser=0;
  Basis.Zaehlerstand_Strom=0;
  

  resetMinMax();  
	usart_write("\n\rSystem Ready.\n\r");

  ntp_timer=30;

	while(1)
	{
     asm("WDR"); //Reset Watchdog
     if (2==strom_markierung){
       //strom_dauer: 61,035Hz oder 16,384ms 
       //Zeit in ms =  strom_dauer * 16,384ms
       //Zhler: 75 Umdrehungen je KWh
       //bei 1KW dauert 1 Umdrehung 1/75 Stunde, also 3600/75=48s
       //Momentanverbrauch[KW] = 48 / Umlaufzeit[s]
       //Momentanverbrauch[W] = 48000 / Umlaufzeit[s]
       //  2929687,5 = 48000 / 0,016384
       //Momentanverbrauch[W] = 2929687,5 / Umlaufzeittics
       
       watt_momentan = 2929687L / strom_dauer;
       sekundenSeitStromflanke = 0;
       strom_markierung=0;
       
     }
  
  
		eth_get_data();
		
        //Terminalcommandos auswerten
		if (usart_status.usart_ready){
            usart_write("\r\n");
			if(extract_cmd(&usart_rx_buffer[0]))
			{
				usart_write("Ready\r\n\r\n");
			}
			else
			{
				usart_write("ERROR\r\n\r\n");
			}
			usart_status.usart_ready =0;
		}
		
        
    //Empfang von Zeitinformationen
		if(!ntp_timer){
			ntp_timer = NTP_REFRESH;
			ntp_request();
		}
		
           
		//USART Daten fr Telnetanwendung?
		telnetd_send_data();
        
        
  }
		
  return(0);
}

